@@ -25,24 +25,31 @@
 
 #include <string.h>
 
-#include "boxed.h"
-#include "arg.h"
-#include "object.h"
-#include "gjs/jsapi-class.h"
 #include "gjs/jsapi-wrapper.h"
-#include "gjs/mem.h"
-#include "repo.h"
-#include "proxyutils.h"
+#include "js/GCHashTable.h"
+
+#include "arg.h"
+#include "boxed.h"
 #include "function.h"
+#include "gi/gerror.h"
+#include "gjs/jsapi-class.h"
+#include "gjs/mem.h"
 #include "gtype.h"
+#include "object.h"
+#include "proxyutils.h"
+#include "repo.h"
 
 #include <util/log.h>
 
 #include <girepository.h>
 
 struct Boxed {
+    using FieldMap =
+        JS::GCHashMap<JS::Heap<JSString*>, GjsAutoFieldInfo,
+                      js::DefaultHasher<JSString*>, js::SystemAllocPolicy>;
+
     /* prototype info */
-    GIBoxedInfo *info;
+    GIStructInfo* info;
     GType gtype;
     gint zero_args_constructor; /* -1 if none */
     JS::Heap<jsid> zero_args_constructor_name;
@@ -51,7 +58,7 @@ struct Boxed {
 
     /* instance info */
     void *gboxed; /* NULL if we are the prototype and not an instance */
-    GHashTable *field_map;
+    FieldMap* field_map;
 
     guint can_allocate_directly : 1;
     guint allocated_directly : 1;
@@ -59,8 +66,10 @@ struct Boxed {
                                     the reference to the C gboxed */
 };
 
+GJS_USE
 static bool struct_is_simple(GIStructInfo *info);
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool boxed_set_field_from_value(JSContext      *context,
                                        Boxed          *priv,
                                        GIFieldInfo    *field_info,
@@ -70,6 +79,7 @@ extern struct JSClass gjs_boxed_class;
 
 GJS_DEFINE_PRIV_FROM_JS(Boxed, gjs_boxed_class)
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 gjs_define_static_methods(JSContext       *context,
                           JS::HandleObject constructor,
@@ -82,10 +92,9 @@ gjs_define_static_methods(JSContext       *context,
     n_methods = g_struct_info_get_n_methods(boxed_info);
 
     for (i = 0; i < n_methods; i++) {
-        GIFunctionInfo *meth_info;
         GIFunctionInfoFlags flags;
 
-        meth_info = g_struct_info_get_method (boxed_info, i);
+        GjsAutoFunctionInfo meth_info = g_struct_info_get_method(boxed_info, i);
         flags = g_function_info_get_flags (meth_info);
 
         /* Anything that isn't a method we put on the prototype of the
@@ -96,17 +105,16 @@ gjs_define_static_methods(JSContext       *context,
          * like in the near future.
          */
         if (!(flags & GI_FUNCTION_IS_METHOD)) {
-            gjs_define_function(context, constructor, gtype,
-                                (GICallableInfo *)meth_info);
+            if (!gjs_define_function(context, constructor, gtype, meth_info))
+                return false;
         }
-
-        g_base_info_unref((GIBaseInfo*) meth_info);
     }
     return true;
 }
 
 /* The *resolved out parameter, on success, should be false to indicate that id
  * was not resolved; and true if id was resolved. */
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 boxed_resolve(JSContext       *context,
               JS::HandleObject obj,
@@ -133,14 +141,17 @@ boxed_resolve(JSContext       *context,
         return true;
     }
 
-    GjsAutoJSChar name;
-    if (!gjs_get_string_id(context, id, &name)) {
+    JS::UniqueChars name;
+    if (!gjs_get_string_id(context, id, &name))
+        return false;
+    if (!name) {
         *resolved = false;
         return true;
     }
 
     /* We are the prototype, so look for methods and other class properties */
-    GIFunctionInfo *method_info = g_struct_info_find_method(priv->info, name);
+    GjsAutoFunctionInfo method_info =
+        g_struct_info_find_method(priv->info, name.get());
     if (!method_info) {
         *resolved = false;
         return true;
@@ -150,32 +161,28 @@ boxed_resolve(JSContext       *context,
 #endif
 
     if (g_function_info_get_flags(method_info) & GI_FUNCTION_IS_METHOD) {
-        const char *method_name = g_base_info_get_name(method_info);
-
         gjs_debug(GJS_DEBUG_GBOXED,
                   "Defining method %s in prototype for %s.%s",
-                  method_name,
+                  method_info.name(),
                   g_base_info_get_namespace( (GIBaseInfo*) priv->info),
                   g_base_info_get_name( (GIBaseInfo*) priv->info));
 
         /* obj is the Boxed prototype */
-        if (!gjs_define_function(context, obj, priv->gtype, method_info)) {
-            g_base_info_unref( (GIBaseInfo*) method_info);
+        if (!gjs_define_function(context, obj, priv->gtype, method_info))
             return false;
-        }
 
         *resolved = true;
     } else {
         *resolved = false;
     }
 
-    g_base_info_unref(method_info);
     return true;
 }
 
 /* Check to see if JS::Value passed in is another Boxed object of the same,
  * and if so, retrieves the Boxed private structure for it.
  */
+GJS_USE
 static bool
 boxed_get_copy_source(JSContext *context,
                       Boxed     *priv,
@@ -216,20 +223,28 @@ boxed_new_direct(Boxed       *priv)
  * to do n O(n) lookups, so put put the fields into a hash table and store it on proto->priv
  * for fast lookup. 
  */
-static GHashTable *
-get_field_map(GIStructInfo *struct_info)
-{
-    GHashTable *result;
+GJS_JSAPI_RETURN_CONVENTION
+static Boxed::FieldMap* get_field_map(JSContext* cx,
+                                      GIStructInfo* struct_info) {
     int n_fields;
     int i;
 
-    result = g_hash_table_new_full(g_str_hash, g_str_equal,
-                                   NULL, (GDestroyNotify)g_base_info_unref);
+    auto* result = new Boxed::FieldMap();
     n_fields = g_struct_info_get_n_fields(struct_info);
+    if (!result->init(n_fields)) {
+        JS_ReportOutOfMemory(cx);
+        return nullptr;
+    }
 
     for (i = 0; i < n_fields; i++) {
-        GIFieldInfo *field_info = g_struct_info_get_field(struct_info, i);
-        g_hash_table_insert(result, (char *)g_base_info_get_name((GIBaseInfo *)field_info), field_info);
+        GjsAutoFieldInfo field_info = g_struct_info_get_field(struct_info, i);
+
+        // We get the string as a jsid later, which is interned. We intern the
+        // string here as well, so it will be the same string pointer
+        JS::RootedString name(cx, JS_NewStringCopyZ(cx, field_info.name()));
+        JSString* atom = JS_AtomizeAndPinJSString(cx, name);
+
+        result->putNewInfallible(atom, std::move(field_info));
     }
 
     return result;
@@ -239,6 +254,7 @@ get_field_map(GIStructInfo *struct_info)
  * properties to set as fieds of the object. We don't require that every field
  * of the object be set.
  */
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 boxed_init_from_props(JSContext   *context,
                       JSObject    *obj,
@@ -260,23 +276,26 @@ boxed_init_from_props(JSContext   *context,
     }
 
     if (!priv->field_map)
-        priv->field_map = get_field_map(priv->info);
+        priv->field_map = get_field_map(context, priv->info);
+    if (!priv->field_map)
+        return false;
 
     JS::RootedValue value(context);
     for (ix = 0, length = ids.length(); ix < length; ix++) {
-        GIFieldInfo *field_info;
-        GjsAutoJSChar name;
-
-        if (!gjs_get_string_id(context, ids[ix], &name))
-            return false;
-
-        field_info = (GIFieldInfo *) g_hash_table_lookup(priv->field_map, name);
-        if (field_info == NULL) {
-            gjs_throw(context, "No field %s on boxed type %s",
-                      name.get(), g_base_info_get_name((GIBaseInfo *)priv->info));
+        if (!JSID_IS_STRING(ids[ix])) {
+            gjs_throw(context, "Fields hash contained a non-string field");
             return false;
         }
 
+        auto entry = priv->field_map->lookup(JSID_TO_STRING(ids[ix]));
+        if (!entry) {
+            gjs_throw(context, "No field %s on boxed type %s",
+                      gjs_debug_id(ids[ix]).c_str(),
+                      g_base_info_get_name(priv->info));
+            return false;
+        }
+        GIFieldInfo* field_info = entry->value().get();
+
         /* ids[ix] is reachable because props is rooted, but require_property
          * doesn't know that */
         if (!gjs_object_require_property(context, props, "property list",
@@ -291,6 +310,7 @@ boxed_init_from_props(JSContext   *context,
     return true;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 boxed_invoke_constructor(JSContext             *context,
                          JS::HandleObject       obj,
@@ -299,8 +319,8 @@ boxed_invoke_constructor(JSContext             *context,
 {
     JS::RootedObject js_constructor(context);
 
-    if (!gjs_object_require_property(context, obj, NULL,
-                                     GJS_STRING_CONSTRUCTOR,
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
+    if (!gjs_object_require_property(context, obj, NULL, atoms.constructor(),
                                      &js_constructor))
         return false;
 
@@ -313,6 +333,7 @@ boxed_invoke_constructor(JSContext             *context,
                                    args, args.rval());
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 boxed_new(JSContext             *context,
           JS::HandleObject       obj, /* "this" for constructor */
@@ -322,9 +343,9 @@ boxed_new(JSContext             *context,
     if (priv->gtype == G_TYPE_VARIANT) {
         /* Short-circuit construction for GVariants by calling into the JS packing
            function */
-        JS::HandleId constructor_name =
-            gjs_context_get_const_string(context, GJS_STRING_NEW_INTERNAL);
-        return boxed_invoke_constructor(context, obj, constructor_name, args);
+        const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
+        return boxed_invoke_constructor(context, obj, atoms.new_internal(),
+                                        args);
     }
 
     /* If the structure is registered as a boxed, we can create a new instance by
@@ -365,9 +386,19 @@ boxed_new(JSContext             *context,
          * that is, Namespace.BoxedType, or object.constructor, given that
          * object was created with the right prototype. The ID is traced from
          * the object, so it's OK to create a handle from it. */
-        return boxed_invoke_constructor(context, obj,
-            JS::HandleId::fromMarkedLocation(priv->default_constructor_name.address()),
-            args);
+        JS::HandleId constructor_name = JS::HandleId::fromMarkedLocation(
+            priv->default_constructor_name.address());
+        if (!boxed_invoke_constructor(context, obj, constructor_name, args))
+            return false;
+
+        // Define the expected Error properties
+        if (priv->gtype == G_TYPE_ERROR) {
+            JS::RootedObject gerror(context, &args.rval().toObject());
+            if (!gjs_define_error_properties(context, gerror))
+                return false;
+        }
+
+        return true;
     } else {
         gjs_throw(context, "Unable to construct struct type %s since it has no default constructor and cannot be allocated directly",
                   g_base_info_get_name((GIBaseInfo*) priv->info));
@@ -492,15 +523,15 @@ boxed_finalize(JSFreeOp *fop,
         priv->info = NULL;
     }
 
-    if (priv->field_map) {
-        g_hash_table_destroy(priv->field_map);
-    }
+    if (priv->field_map)
+        delete priv->field_map;
 
     GJS_DEC_COUNTER(boxed);
     priv->~Boxed();
     g_slice_free(Boxed, priv);
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static GIFieldInfo *
 get_field_info(JSContext *cx,
                Boxed     *priv,
@@ -516,6 +547,7 @@ get_field_info(JSContext *cx,
     return field_info;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 get_nested_interface_object(JSContext             *context,
                             JSObject              *parent_obj,
@@ -538,9 +570,11 @@ get_nested_interface_object(JSContext             *context,
         return false;
     }
 
-    JS::RootedObject proto(context,
-                           gjs_lookup_generic_prototype(context,
-                                                        (GIBoxedInfo*) interface_info));
+    JS::RootedObject proto(
+        context, gjs_lookup_generic_prototype(context, interface_info));
+
+    if (!proto)
+        return false;
     proto_priv = priv_from_js(context, proto);
 
     offset = g_field_info_get_offset (field_info);
@@ -554,7 +588,7 @@ get_nested_interface_object(JSContext             *context,
     priv = g_slice_new0(Boxed);
     new (priv) Boxed();
     JS_SetPrivate(obj, priv);
-    priv->info = (GIBoxedInfo*) interface_info;
+    priv->info = interface_info;
     g_base_info_ref( (GIBaseInfo*) priv->info);
     priv->gtype = g_registered_type_info_get_g_type ((GIRegisteredTypeInfo*) interface_info);
     priv->can_allocate_directly = proto_priv->can_allocate_directly;
@@ -572,6 +606,7 @@ get_nested_interface_object(JSContext             *context,
     return true;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 boxed_field_getter(JSContext *context,
                    unsigned   argc,
@@ -638,6 +673,7 @@ out:
     return success;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 set_nested_interface_object (JSContext      *context,
                              Boxed          *parent_priv,
@@ -658,9 +694,11 @@ set_nested_interface_object (JSContext      *context,
         return false;
     }
 
-    JS::RootedObject proto(context,
-                           gjs_lookup_generic_prototype(context,
-                                                        (GIBoxedInfo*) interface_info));
+    JS::RootedObject proto(
+        context, gjs_lookup_generic_prototype(context, interface_info));
+
+    if (!proto)
+        return false;
     proto_priv = priv_from_js(context, proto);
 
     /* If we can't directly copy from the source object we need
@@ -687,6 +725,7 @@ set_nested_interface_object (JSContext      *context,
     return true;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 boxed_set_field_from_value(JSContext      *context,
                            Boxed          *priv,
@@ -741,16 +780,20 @@ boxed_set_field_from_value(JSContext      *context,
     success = true;
 
 out:
-    if (need_release)
-        gjs_g_argument_release (context, GI_TRANSFER_NOTHING,
-                                type_info,
-                                &arg);
+    if (need_release) {
+        JS::AutoSaveExceptionState saved_exc(context);
+        if (!gjs_g_argument_release(context, GI_TRANSFER_NOTHING, type_info,
+                                    &arg))
+            gjs_log_exception(context);
+        saved_exc.restore();
+    }
 
     g_base_info_unref ((GIBaseInfo *)type_info);
 
     return success;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 boxed_field_setter(JSContext *cx,
                    unsigned   argc,
@@ -785,6 +828,7 @@ out:
     return success;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 define_boxed_class_fields(JSContext       *cx,
                           Boxed           *priv,
@@ -825,6 +869,7 @@ define_boxed_class_fields(JSContext       *cx,
     return true;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 to_string_func(JSContext *context,
                unsigned   argc,
@@ -848,6 +893,8 @@ boxed_trace(JSTracer *tracer,
                         "Boxed::zero_args_constructor_name");
     JS::TraceEdge<jsid>(tracer, &priv->default_constructor_name,
                         "Boxed::default_constructor_name");
+    if (priv->field_map)
+        priv->field_map->trace(tracer);
 }
 
 /* The bizarre thing about this vtable is that it applies to both
@@ -887,6 +934,7 @@ JSFunctionSpec gjs_boxed_proto_funcs[] = {
     JS_FN("toString", to_string_func, 0, 0),
     JS_FS_END};
 
+GJS_USE
 static bool
 type_can_be_allocated_directly(GITypeInfo *type_info)
 {
@@ -979,6 +1027,7 @@ type_can_be_allocated_directly(GITypeInfo *type_info)
  * type that we know how to assign to. If so, then we can allocate and free
  * instances without needing a constructor.
  */
+GJS_USE
 static bool
 struct_is_simple(GIStructInfo *info)
 {
@@ -1053,7 +1102,8 @@ boxed_fill_prototype_info(JSContext *context,
                 if (priv->default_constructor < 0 &&
                     strcmp(g_base_info_get_name ((GIBaseInfo*) func_info), "new") == 0) {
                     priv->default_constructor = i;
-                    priv->default_constructor_name = gjs_context_get_const_string(context, GJS_STRING_NEW);
+                    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
+                    priv->default_constructor_name = atoms.new_();
                 }
             }
 
@@ -1071,11 +1121,8 @@ boxed_fill_prototype_info(JSContext *context,
     }
 }
 
-void
-gjs_define_boxed_class(JSContext       *context,
-                       JS::HandleObject in_object,
-                       GIBoxedInfo     *info)
-{
+bool gjs_define_boxed_class(JSContext* context, JS::HandleObject in_object,
+                            GIStructInfo* info) {
     const char *constructor_name;
     JS::RootedObject prototype(context), constructor(context);
     Boxed *priv;
@@ -1103,8 +1150,7 @@ gjs_define_boxed_class(JSContext       *context,
                                 NULL,
                                 &prototype,
                                 &constructor)) {
-        gjs_log_exception(context);
-        g_error("Can't init class %s", constructor_name);
+        return false;
     }
 
     GJS_INC_COUNTER(boxed);
@@ -1123,22 +1169,28 @@ gjs_define_boxed_class(JSContext       *context,
 
     priv->can_allocate_directly = struct_is_simple (priv->info);
 
-    define_boxed_class_fields (context, priv, prototype);
-    gjs_define_static_methods (context, constructor, priv->gtype, priv->info);
+    if (!define_boxed_class_fields(context, priv, prototype) ||
+        !gjs_define_static_methods(context, constructor, priv->gtype,
+                                   priv->info))
+        return false;
+
+    if (priv->gtype == G_TYPE_ERROR &&
+        !JS_DefineFunction(context, prototype, "toString", gjs_gerror_to_string,
+                           0, GJS_MODULE_PROP_FLAGS))
+        return false;
 
     JS::RootedObject gtype_obj(context,
         gjs_gtype_create_gtype_wrapper(context, priv->gtype));
-    JS_DefineProperty(context, constructor, "$gtype", gtype_obj,
-                      JSPROP_PERMANENT);
+    if (!gtype_obj)
+        return false;
+
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
+    return JS_DefinePropertyById(context, constructor, atoms.gtype(), gtype_obj,
+                                 JSPROP_PERMANENT);
 }
 
-JSObject*
-gjs_boxed_from_c_struct(JSContext             *context,
-                        GIStructInfo          *info,
-                        void                  *gboxed,
-                        GjsBoxedCreationFlags  flags)
-{
-    JSObject *obj;
+JSObject* gjs_boxed_from_c_struct(JSContext* cx, GIStructInfo* info,
+                                  void* gboxed, GjsBoxedCreationFlags flags) {
     Boxed *priv;
     Boxed *proto_priv;
 
@@ -1149,10 +1201,15 @@ gjs_boxed_from_c_struct(JSContext             *context,
                       "Wrapping struct %s %p with JSObject",
                       g_base_info_get_name((GIBaseInfo *)info), gboxed);
 
-    JS::RootedObject proto(context, gjs_lookup_generic_prototype(context, info));
-    proto_priv = priv_from_js(context, proto);
+    JS::RootedObject proto(cx, gjs_lookup_generic_prototype(cx, info));
+    if (!proto)
+        return nullptr;
+    proto_priv = priv_from_js(cx, proto);
 
-    obj = JS_NewObjectWithGivenProto(context, JS_GetClass(proto), proto);
+    JS::RootedObject obj(
+        cx, JS_NewObjectWithGivenProto(cx, JS_GetClass(proto), proto));
+    if (!obj)
+        return nullptr;
 
     GJS_INC_COUNTER(boxed);
     priv = g_slice_new0(Boxed);
@@ -1179,12 +1236,15 @@ gjs_boxed_from_c_struct(JSContext             *context,
             boxed_new_direct(priv);
             memcpy(priv->gboxed, gboxed, g_struct_info_get_size (priv->info));
         } else {
-            gjs_throw(context,
+            gjs_throw(cx,
                       "Can't create a Javascript object for %s; no way to copy",
-                      g_base_info_get_name( (GIBaseInfo*) priv->info));
+                      g_base_info_get_name(priv->info));
         }
     }
 
+    if (priv->gtype == G_TYPE_ERROR && !gjs_define_error_properties(cx, obj))
+        return nullptr;
+
     return obj;
 }
 
