@@ -73,6 +73,7 @@ extern struct JSClass gjs_fundamental_instance_class;
 
 GJS_DEFINE_PRIV_FROM_JS(FundamentalInstance, gjs_fundamental_instance_class)
 
+GJS_USE
 static GQuark
 gjs_fundamental_table_quark (void)
 {
@@ -83,6 +84,7 @@ gjs_fundamental_table_quark (void)
     return val;
 }
 
+GJS_USE
 static GHashTable *
 _ensure_mapping_table(GjsContext *context)
 {
@@ -117,6 +119,7 @@ _fundamental_remove_object(void *native_object)
     g_hash_table_remove(table, native_object);
 }
 
+GJS_USE
 static JSObject *
 _fundamental_lookup_object(void *native_object)
 {
@@ -127,18 +130,21 @@ _fundamental_lookup_object(void *native_object)
 
 /**/
 
+GJS_USE
 static inline bool
 fundamental_is_prototype(Fundamental *priv)
 {
     return (priv->prototype == nullptr);
 }
 
+GJS_USE
 static inline bool
 fundamental_is_prototype(FundamentalInstance *priv)
 {
     return (priv->prototype == nullptr);
 }
 
+GJS_USE
 static inline Fundamental *
 proto_priv_from_js(JSContext       *context,
                    JS::HandleObject obj)
@@ -203,6 +209,7 @@ associate_js_instance_to_fundamental(JSContext       *context,
 /**/
 
 /* Find the first constructor */
+GJS_USE
 static GIFunctionInfo *
 find_fundamental_constructor(JSContext          *context,
                              GIObjectInfo       *info,
@@ -236,6 +243,7 @@ find_fundamental_constructor(JSContext          *context,
 
 /**/
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 fundamental_instance_resolve_interface(JSContext       *context,
                                        JS::HandleObject obj,
@@ -243,7 +251,6 @@ fundamental_instance_resolve_interface(JSContext       *context,
                                        Fundamental     *proto_priv,
                                        const char      *name)
 {
-    GIFunctionInfo *method_info;
     bool ret;
     GType *interfaces;
     guint n_interfaces;
@@ -252,37 +259,23 @@ fundamental_instance_resolve_interface(JSContext       *context,
     ret = true;
     interfaces = g_type_interfaces(proto_priv->gtype, &n_interfaces);
     for (i = 0; i < n_interfaces; i++) {
-        GIBaseInfo *base_info;
-        GIInterfaceInfo *iface_info;
+        GjsAutoInterfaceInfo iface_info =
+            g_irepository_find_by_gtype(nullptr, interfaces[i]);
 
-        base_info = g_irepository_find_by_gtype(g_irepository_get_default(),
-                                                interfaces[i]);
-
-        if (base_info == NULL)
+        if (!iface_info)
             continue;
 
-        /* An interface GType ought to have interface introspection info */
-        g_assert(g_base_info_get_type(base_info) == GI_INFO_TYPE_INTERFACE);
+        GjsAutoFunctionInfo method_info =
+            g_interface_info_find_method(iface_info, name);
 
-        iface_info = (GIInterfaceInfo *) base_info;
-
-        method_info = g_interface_info_find_method(iface_info, name);
-
-        g_base_info_unref(base_info);
-
-
-        if (method_info != NULL) {
-            if (g_function_info_get_flags (method_info) & GI_FUNCTION_IS_METHOD) {
-                if (gjs_define_function(context, obj,
-                                        proto_priv->gtype,
-                                        (GICallableInfo *) method_info)) {
-                    *resolved = true;
-                } else {
-                    ret = false;
-                }
+        if (method_info &&
+            g_function_info_get_flags(method_info) & GI_FUNCTION_IS_METHOD) {
+            if (gjs_define_function(context, obj, proto_priv->gtype,
+                                    method_info)) {
+                *resolved = true;
+            } else {
+                ret = false;
             }
-
-            g_base_info_unref((GIBaseInfo *) method_info);
         }
     }
 
@@ -294,6 +287,7 @@ fundamental_instance_resolve_interface(JSContext       *context,
  * The *resolved out parameter, on success, should be false to indicate that id
  * was not resolved; and true if id was resolved.
  */
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 fundamental_instance_resolve(JSContext       *context,
                              JS::HandleObject obj,
@@ -322,66 +316,57 @@ fundamental_instance_resolve(JSContext       *context,
         return true;
     }
 
-    GjsAutoJSChar name;
-    if (!gjs_get_string_id(context, id, &name)) {
+    JS::UniqueChars name;
+    if (!gjs_get_string_id(context, id, &name))
+        return false;
+    if (!name) {
         *resolved = false;
         return true; /* not resolved, but no error */
     }
 
     /* We are the prototype, so look for methods and other class properties */
     Fundamental *proto_priv = (Fundamental *) priv;
-    GIFunctionInfo *method_info;
-
-    method_info = g_object_info_find_method((GIStructInfo*) proto_priv->info,
-                                            name);
-
-    if (method_info != NULL) {
-        const char *method_name;
+    GjsAutoFunctionInfo method_info =
+        g_object_info_find_method(proto_priv->info, name.get());
 
+    if (method_info) {
 #if GJS_VERBOSE_ENABLE_GI_USAGE
-        _gjs_log_info_usage((GIBaseInfo *) method_info);
+        _gjs_log_info_usage(method_info);
 #endif
         if (g_function_info_get_flags (method_info) & GI_FUNCTION_IS_METHOD) {
-            method_name = g_base_info_get_name((GIBaseInfo *) method_info);
-
             /* we do not define deprecated methods in the prototype */
-            if (g_base_info_is_deprecated((GIBaseInfo *) method_info)) {
+            if (g_base_info_is_deprecated(method_info)) {
                 gjs_debug(GJS_DEBUG_GFUNDAMENTAL,
                           "Ignoring definition of deprecated method %s in prototype %s.%s",
-                          method_name,
+                          method_info.name(),
                           g_base_info_get_namespace((GIBaseInfo *) proto_priv->info),
                           g_base_info_get_name((GIBaseInfo *) proto_priv->info));
-                g_base_info_unref((GIBaseInfo *) method_info);
                 *resolved = false;
                 return true;
             }
 
             gjs_debug(GJS_DEBUG_GFUNDAMENTAL,
                       "Defining method %s in prototype for %s.%s",
-                      method_name,
+                      method_info.name(),
                       g_base_info_get_namespace((GIBaseInfo *) proto_priv->info),
                       g_base_info_get_name((GIBaseInfo *) proto_priv->info));
 
-            if (gjs_define_function(context, obj, proto_priv->gtype,
-                                    method_info) == NULL) {
-                g_base_info_unref((GIBaseInfo *) method_info);
+            if (!gjs_define_function(context, obj, proto_priv->gtype,
+                                     method_info))
                 return false;
-            }
 
             *resolved = true;
         }
-
-        g_base_info_unref((GIBaseInfo *) method_info);
     } else {
         *resolved = false;
     }
 
-    bool status =
-        fundamental_instance_resolve_interface(context, obj, resolved,
-                                               proto_priv, name);
+    bool status = fundamental_instance_resolve_interface(
+        context, obj, resolved, proto_priv, name.get());
     return status;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 fundamental_invoke_constructor(FundamentalInstance        *priv,
                                JSContext                  *context,
@@ -391,8 +376,8 @@ fundamental_invoke_constructor(FundamentalInstance        *priv,
 {
     JS::RootedObject js_constructor(context);
 
-    if (!gjs_object_require_property(context, obj, NULL,
-                                     GJS_STRING_CONSTRUCTOR,
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
+    if (!gjs_object_require_property(context, obj, NULL, atoms.constructor(),
                                      &js_constructor) ||
         priv->prototype->constructor_name.get() == JSID_VOID) {
         gjs_throw (context,
@@ -495,6 +480,7 @@ fundamental_finalize(JSFreeOp  *fop,
     }
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 to_string_func(JSContext *context,
                unsigned   argc,
@@ -571,6 +557,7 @@ static JSFunctionSpec gjs_fundamental_instance_proto_funcs[] = {
     JS_FN("toString", to_string_func, 0, 0),
     JS_FS_END};
 
+GJS_JSAPI_RETURN_CONVENTION
 static JSObject *
 gjs_lookup_fundamental_prototype(JSContext    *context,
                                  GIObjectInfo *info,
@@ -600,30 +587,37 @@ gjs_lookup_fundamental_prototype(JSContext    *context,
            we need to define it first.
         */
         JS::RootedObject ignored(context);
-        gjs_define_fundamental_class(context, in_object, info, &constructor,
-                                     &ignored);
+        if (!gjs_define_fundamental_class(context, in_object, info,
+                                          &constructor, &ignored))
+            return nullptr;
     } else {
-        if (G_UNLIKELY (!value.isObject()))
+        if (G_UNLIKELY(!value.isObject())) {
+            gjs_throw(context,
+                      "Fundamental constructor was not an object, it was a %s",
+                      JS::InformalValueTypeName(value));
             return NULL;
+        }
 
         constructor = &value.toObject();
     }
 
     g_assert(constructor);
 
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
     JS::RootedObject prototype(context);
     if (!gjs_object_require_property(context, constructor, "constructor object",
-                                     GJS_STRING_PROTOTYPE, &prototype))
+                                     atoms.prototype(), &prototype))
         return NULL;
 
     return prototype;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static JSObject*
 gjs_lookup_fundamental_prototype_from_gtype(JSContext *context,
                                             GType      gtype)
 {
-    GjsAutoInfo<GIObjectInfo> info;
+    GjsAutoObjectInfo info;
 
     /* A given gtype might not have any definition in the introspection
      * data. If that's the case, try to look for a definition of any of the
@@ -728,8 +722,12 @@ gjs_define_fundamental_class(JSContext              *context,
 
     JS::RootedObject gtype_obj(context,
         gjs_gtype_create_gtype_wrapper(context, gtype));
-    return JS_DefineProperty(context, constructor, "$gtype", gtype_obj,
-                             JSPROP_PERMANENT);
+    if (!gtype_obj)
+        return false;
+
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
+    return JS_DefinePropertyById(context, constructor, atoms.gtype(), gtype_obj,
+                                 JSPROP_PERMANENT);
 }
 
 JSObject*
