@@ -72,7 +72,6 @@ static_assert(sizeof(ObjectInstance) <= 88,
 std::stack<JS::PersistentRootedObject> ObjectInstance::object_init_list{};
 static bool weak_pointer_callback = false;
 ObjectInstance *ObjectInstance::wrapped_gobject_list = nullptr;
-JS::PersistentRootedSymbol ObjectInstance::hook_up_vfunc_root;
 
 extern struct JSClass gjs_object_instance_class;
 GJS_DEFINE_PRIV_FROM_JS(ObjectBase, gjs_object_instance_class)
@@ -82,6 +81,7 @@ G_DEFINE_QUARK(gjs::custom-type, ObjectBase::custom_type)
 G_DEFINE_QUARK(gjs::custom-property, ObjectBase::custom_property)
 // clang-format on
 
+GJS_USE
 static GQuark
 gjs_object_priv_quark (void)
 {
@@ -274,31 +274,30 @@ ObjectInstance::unset_object_qdata(void)
 
 GParamSpec* ObjectPrototype::find_param_spec_from_id(JSContext* cx,
                                                      JS::HandleString key) {
-    char *gname;
-
     /* First check for the ID in the cache */
     auto entry = m_property_cache.lookupForAdd(key);
     if (entry)
-        return entry->value().get();
+        return entry->value();
 
-    GjsAutoJSChar js_prop_name;
+    JS::UniqueChars js_prop_name;
     if (!gjs_string_to_utf8(cx, JS::StringValue(key), &js_prop_name))
         return nullptr;
 
-    gname = gjs_hyphen_from_camel(js_prop_name);
-    GObjectClass *gobj_class = G_OBJECT_CLASS(g_type_class_ref(m_gtype));
-    GjsAutoParam param_spec = g_object_class_find_property(gobj_class, gname);
-    g_type_class_unref(gobj_class);
-    g_free(gname);
+    GjsAutoChar gname = gjs_hyphen_from_camel(js_prop_name.get());
+    GjsAutoTypeClass<GObjectClass> gobj_class(m_gtype);
+    GParamSpec* pspec = g_object_class_find_property(gobj_class, gname);
+    GjsAutoParam param_spec(pspec, GjsAutoTakeOwnership());
 
     if (!param_spec) {
-        _gjs_proxy_throw_nonexistent_field(cx, m_gtype, js_prop_name);
+        _gjs_proxy_throw_nonexistent_field(cx, m_gtype, js_prop_name.get());
         return nullptr;
     }
 
-    if (!m_property_cache.add(entry, key, std::move(param_spec)))
-        g_error("Out of memory adding param spec to cache");
-    return entry->value().get();  /* owned by property cache */
+    if (!m_property_cache.add(entry, key, std::move(param_spec))) {
+        JS_ReportOutOfMemory(cx);
+        return nullptr;
+    }
+    return pspec; /* owned by property cache */
 }
 
 /* Gets the ObjectPrototype corresponding to obj.prototype. Cannot return null,
@@ -386,8 +385,10 @@ ObjectInstance::prop_getter_impl(JSContext             *cx,
     if (g_param_spec_get_qdata(param, ObjectInstance::custom_property_quark()))
         return true;
 
-    if ((param->flags & G_PARAM_READABLE) == 0)
+    if ((param->flags & G_PARAM_READABLE) == 0) {
+        rval.setUndefined();
         return true;
+    }
 
     gjs_debug_jsprop(GJS_DEBUG_GOBJECT, "Accessing GObject property %s",
                      param->name);
@@ -403,13 +404,12 @@ ObjectInstance::prop_getter_impl(JSContext             *cx,
     return true;
 }
 
-static GjsAutoInfo<GIFieldInfo>
-lookup_field_info(GIObjectInfo *info,
-                  const char   *name)
-{
+GJS_USE
+static GjsAutoFieldInfo lookup_field_info(GIObjectInfo* info,
+                                          const char* name) {
     int n_fields = g_object_info_get_n_fields(info);
     int ix;
-    GjsAutoInfo<GIFieldInfo> retval;
+    GjsAutoFieldInfo retval;
 
     for (ix = 0; ix < n_fields; ix++) {
         retval = g_object_info_get_field(info, ix);
@@ -431,19 +431,21 @@ GIFieldInfo* ObjectPrototype::find_field_info_from_id(JSContext* cx,
     if (entry)
         return entry->value().get();
 
-    GjsAutoJSChar js_prop_name;
+    JS::UniqueChars js_prop_name;
     if (!gjs_string_to_utf8(cx, JS::StringValue(key), &js_prop_name))
         return nullptr;
 
-    GjsAutoInfo<GIFieldInfo> field = lookup_field_info(m_info, js_prop_name);
+    GjsAutoFieldInfo field = lookup_field_info(m_info, js_prop_name.get());
 
     if (!field) {
-        _gjs_proxy_throw_nonexistent_field(cx, m_gtype, js_prop_name);
+        _gjs_proxy_throw_nonexistent_field(cx, m_gtype, js_prop_name.get());
         return nullptr;
     }
 
-    if (!m_field_cache.add(entry, key, std::move(field)))
-        g_error("Out of memory adding field info to cache");
+    if (!m_field_cache.add(entry, key, std::move(field))) {
+        JS_ReportOutOfMemory(cx);
+        return nullptr;
+    }
     return entry->value().get();  /* owned by field cache */
 }
 
@@ -483,7 +485,7 @@ ObjectInstance::field_getter_impl(JSContext             *cx,
     gjs_debug_jsprop(GJS_DEBUG_GOBJECT, "Overriding %s with GObject field",
                      gjs_debug_string(name).c_str());
 
-    GjsAutoInfo<GITypeInfo> type = g_field_info_get_type(field);
+    GjsAutoTypeInfo type = g_field_info_get_type(field);
     tag = g_type_info_get_tag(type);
     if (tag == GI_TYPE_TAG_ARRAY ||
         tag == GI_TYPE_TAG_INTERFACE ||
@@ -637,21 +639,20 @@ bool ObjectPrototype::is_vfunc_unchanged(GIVFuncInfo* info) {
     return addr1 == addr2;
 }
 
-static GjsAutoInfo<GIVFuncInfo>
-find_vfunc_on_parents(GIObjectInfo *info,
-                      const char   *name,
-                      bool         *out_defined_by_parent)
-{
+GJS_USE
+static GjsAutoVFuncInfo find_vfunc_on_parents(GIObjectInfo* info,
+                                              const char* name,
+                                              bool* out_defined_by_parent) {
     bool defined_by_parent = false;
 
     /* ref the first info so that we don't destroy
      * it when unrefing parents later */
-    GjsAutoInfo<GIObjectInfo> parent = g_base_info_ref(info);
+    GjsAutoObjectInfo parent = g_base_info_ref(info);
 
     /* Since it isn't possible to override a vfunc on
      * an interface without reimplementing it, we don't need
      * to search the parent types when looking for a vfunc. */
-    GjsAutoInfo<GIVFuncInfo> vfunc =
+    GjsAutoVFuncInfo vfunc =
         g_object_info_find_vfunc_using_interfaces(parent, name, nullptr);
     while (!vfunc && parent) {
         parent = g_object_info_get_parent(parent);
@@ -668,8 +669,8 @@ find_vfunc_on_parents(GIObjectInfo *info,
 }
 
 /* Taken from GLib */
-static void canonicalize_key(char* key) {
-    for (char* p = key; *p != 0; p++) {
+static void canonicalize_key(const GjsAutoChar& key) {
+    for (char* p = key.get(); *p != 0; p++) {
         char c = *p;
 
         if (c != '-' && (c < '0' || c > '9') && (c < 'A' || c > 'Z') &&
@@ -679,10 +680,11 @@ static void canonicalize_key(char* key) {
 }
 
 /* @name must already be canonicalized */
+GJS_USE
 static bool is_ginterface_property_name(GIInterfaceInfo* info,
                                         const char* name) {
     int n_props = g_interface_info_get_n_properties(info);
-    GjsAutoInfo<GIPropertyInfo> prop_info;
+    GjsAutoPropertyInfo prop_info;
 
     for (int ix = 0; ix < n_props; ix++) {
         prop_info = g_interface_info_get_property(info, ix);
@@ -691,10 +693,7 @@ static bool is_ginterface_property_name(GIInterfaceInfo* info,
         prop_info.reset();
     }
 
-    if (!prop_info)
-        return false;
-
-    return g_property_info_get_flags(prop_info) & G_PARAM_READABLE;
+    return !!prop_info;
 }
 
 bool ObjectPrototype::lazy_define_gobject_property(JSContext* cx,
@@ -737,22 +736,18 @@ bool ObjectPrototype::resolve_no_info(JSContext* cx, JS::HandleObject obj,
 
     GjsAutoChar canonical_name;
     if (resolve_props == ConsiderMethodsAndProperties) {
-        char* canonical_name_unowned = gjs_hyphen_from_camel(name);
-        canonicalize_key(canonical_name_unowned);
-        canonical_name.reset(canonical_name_unowned);
+        canonical_name = gjs_hyphen_from_camel(name);
+        canonicalize_key(canonical_name);
     }
 
     GType *interfaces = g_type_interfaces(m_gtype, &n_interfaces);
     for (i = 0; i < n_interfaces; i++) {
-        GjsAutoInfo<GIInterfaceInfo> iface_info =
+        GjsAutoInterfaceInfo iface_info =
             g_irepository_find_by_gtype(nullptr, interfaces[i]);
         if (!iface_info)
             continue;
 
-        /* An interface GType ought to have interface introspection info */
-        g_assert(iface_info.type() == GI_INFO_TYPE_INTERFACE);
-
-        GjsAutoInfo<GIFunctionInfo> method_info =
+        GjsAutoFunctionInfo method_info =
             g_interface_info_find_method(iface_info, name);
         if (method_info != NULL) {
             if (g_function_info_get_flags (method_info) & GI_FUNCTION_IS_METHOD) {
@@ -784,6 +779,7 @@ bool ObjectPrototype::resolve_no_info(JSContext* cx, JS::HandleObject obj,
     return true;
 }
 
+GJS_USE
 static bool
 is_gobject_property_name(GIObjectInfo *info,
                          const char   *name)
@@ -791,9 +787,9 @@ is_gobject_property_name(GIObjectInfo *info,
     int n_props = g_object_info_get_n_properties(info);
     int n_ifaces = g_object_info_get_n_interfaces(info);
     int ix;
-    GjsAutoInfo<GIPropertyInfo> prop_info;
+    GjsAutoPropertyInfo prop_info;
 
-    char *canonical_name = gjs_hyphen_from_camel(name);
+    GjsAutoChar canonical_name = gjs_hyphen_from_camel(name);
     canonicalize_key(canonical_name);
 
     for (ix = 0; ix < n_props; ix++) {
@@ -805,21 +801,15 @@ is_gobject_property_name(GIObjectInfo *info,
 
     if (!prop_info) {
         for (ix = 0; ix < n_ifaces; ix++) {
-            GjsAutoInfo<GIInterfaceInfo> iface_info =
+            GjsAutoInterfaceInfo iface_info =
                 g_object_info_get_interface(info, ix);
-            if (is_ginterface_property_name(iface_info, canonical_name)) {
-                g_free(canonical_name);
+            if (is_ginterface_property_name(iface_info, canonical_name))
                 return true;
-            }
         }
+        return false;
     }
 
-    g_free(canonical_name);
-
-    if (!prop_info)
-        return false;
-
-    return g_property_info_get_flags(prop_info) & G_PARAM_READABLE;
+    return true;
 }
 
 bool ObjectBase::resolve(JSContext* cx, JS::HandleObject obj, JS::HandleId id,
@@ -848,12 +838,32 @@ bool ObjectBase::resolve(JSContext* cx, JS::HandleObject obj, JS::HandleId id,
     return priv->to_prototype()->resolve_impl(cx, obj, id, resolved);
 }
 
+/* The JSResolveOp for an instance is called for every property not
+ * defined, even if it's one of the functions or properties we're
+ * adding to the proto manually.
+ */
+static bool name_is_overridden(jsid name, const GjsAtoms& atoms) {
+    // Keep this list in sync with gjs_object_instance_proto_props and
+    // gjs_object_instance_proto_funcs. However, explicitly do not include
+    // connect() in it, because there are a few cases where the lazy property
+    // should override the predefined one, such as Gio.Cancellable.connect().
+    return name == atoms.init() || name == atoms.connect_after() ||
+           name == atoms.emit() || name == atoms.to_string();
+}
+
 bool ObjectPrototype::resolve_impl(JSContext* context, JS::HandleObject obj,
                                    JS::HandleId id, bool* resolved) {
     debug_jsprop("Resolve hook", id, obj);
 
-    GjsAutoJSChar name;
-    if (!gjs_get_string_id(context, id, &name)) {
+    if (name_is_overridden(id, GjsContextPrivate::atoms(context))) {
+        *resolved = false;
+        return true;
+    }
+
+    JS::UniqueChars name;
+    if (!gjs_get_string_id(context, id, &name))
+        return false;
+    if (!name) {
         *resolved = false;
         return true;  /* not resolved, but no error */
     }
@@ -862,10 +872,10 @@ bool ObjectPrototype::resolve_impl(JSContext* context, JS::HandleObject obj,
      * we need to look at exposing interfaces. Look up our interfaces through
      * GType data, and then hope that *those* are introspectable. */
     if (is_custom_js_class())
-        return resolve_no_info(context, obj, id, resolved, name,
+        return resolve_no_info(context, obj, id, resolved, name.get(),
                                ConsiderMethodsAndProperties);
 
-    if (g_str_has_prefix (name, "vfunc_")) {
+    if (g_str_has_prefix(name.get(), "vfunc_")) {
         /* The only time we find a vfunc info is when we're the base
          * class that defined the vfunc. If we let regular prototype
          * chaining resolve this, we'd have the implementation for the base's
@@ -882,9 +892,8 @@ bool ObjectPrototype::resolve_impl(JSContext* context, JS::HandleObject obj,
 
         const char *name_without_vfunc_ = &(name[6]);  /* lifetime tied to name */
         bool defined_by_parent;
-        GjsAutoInfo<GIVFuncInfo> vfunc = find_vfunc_on_parents(m_info,
-                                                               name_without_vfunc_,
-                                                               &defined_by_parent);
+        GjsAutoVFuncInfo vfunc = find_vfunc_on_parents(
+            m_info, name_without_vfunc_, &defined_by_parent);
         if (vfunc != NULL) {
 
             /* In the event that the vfunc is unchanged, let regular
@@ -894,7 +903,9 @@ bool ObjectPrototype::resolve_impl(JSContext* context, JS::HandleObject obj,
                 return true;
             }
 
-            gjs_define_function(context, obj, m_gtype, vfunc);
+            if (!gjs_define_function(context, obj, m_gtype, vfunc))
+                return false;
+
             *resolved = true;
             return true;
         }
@@ -903,10 +914,11 @@ bool ObjectPrototype::resolve_impl(JSContext* context, JS::HandleObject obj,
          * method resolution. */
     }
 
-    if (is_gobject_property_name(m_info, name))
-        return lazy_define_gobject_property(context, obj, id, resolved, name);
+    if (is_gobject_property_name(m_info, name.get()))
+        return lazy_define_gobject_property(context, obj, id, resolved,
+                                            name.get());
 
-    GjsAutoInfo<GIFieldInfo> field_info = lookup_field_info(m_info, name);
+    GjsAutoFieldInfo field_info = lookup_field_info(m_info, name.get());
     if (field_info) {
         bool found = false;
         if (!JS_AlreadyHasOwnPropertyById(context, obj, id, &found))
@@ -923,10 +935,10 @@ bool ObjectPrototype::resolve_impl(JSContext* context, JS::HandleObject obj,
             flags |= JSPROP_READONLY;
 
         JS::RootedValue private_id(context, JS::StringValue(JSID_TO_STRING(id)));
-        if (!gjs_define_property_dynamic(context, obj, name, "gobject_field",
-                                         &ObjectBase::field_getter,
-                                         &ObjectBase::field_setter, private_id,
-                                         flags))
+        if (!gjs_define_property_dynamic(
+                context, obj, name.get(), "gobject_field",
+                &ObjectBase::field_getter, &ObjectBase::field_setter,
+                private_id, flags))
             return false;
 
         *resolved = true;
@@ -944,8 +956,8 @@ bool ObjectPrototype::resolve_impl(JSContext* context, JS::HandleObject obj,
      * introduces the iface)
      */
 
-    GjsAutoInfo<GIFunctionInfo> method_info =
-        g_object_info_find_method_using_interfaces(m_info, name, nullptr);
+    GjsAutoFunctionInfo method_info =
+        g_object_info_find_method_using_interfaces(m_info, name.get(), nullptr);
 
     /**
      * Search through any interfaces implemented by the GType;
@@ -953,11 +965,11 @@ bool ObjectPrototype::resolve_impl(JSContext* context, JS::HandleObject obj,
      * https://bugzilla.gnome.org/show_bug.cgi?id=632922
      */
     if (!method_info)
-        return resolve_no_info(context, obj, id, resolved, name,
+        return resolve_no_info(context, obj, id, resolved, name.get(),
                                ConsiderOnlyMethods);
 
 #if GJS_VERBOSE_ENABLE_GI_USAGE
-    _gjs_log_info_usage((GIBaseInfo*) method_info);
+    _gjs_log_info_usage(method_info);
 #endif
 
     if (g_function_info_get_flags (method_info) & GI_FUNCTION_IS_METHOD) {
@@ -1001,7 +1013,7 @@ bool ObjectPrototype::new_enumerate_impl(JSContext* cx, JS::HandleObject obj,
     GType* interfaces = g_type_interfaces(gtype(), &n_interfaces);
 
     for (unsigned k = 0; k < n_interfaces; k++) {
-        GjsAutoInfo<GIInterfaceInfo> iface_info =
+        GjsAutoInterfaceInfo iface_info =
             g_irepository_find_by_gtype(nullptr, interfaces[k]);
 
         if (!iface_info) {
@@ -1011,14 +1023,15 @@ bool ObjectPrototype::new_enumerate_impl(JSContext* cx, JS::HandleObject obj,
         // Methods
         int n_methods = g_interface_info_get_n_methods(iface_info);
         for (int i = 0; i < n_methods; i++) {
-            GjsAutoInfo<GIFunctionInfo> meth_info =
+            GjsAutoFunctionInfo meth_info =
                 g_interface_info_get_method(iface_info, i);
             GIFunctionInfoFlags flags = g_function_info_get_flags(meth_info);
 
             if (flags & GI_FUNCTION_IS_METHOD) {
                 const char* name = meth_info.name();
                 if (!properties.append(gjs_intern_string_to_id(cx, name))) {
-                    g_error("Unable to append to vector");
+                    JS_ReportOutOfMemory(cx);
+                    return false;
                 }
             }
         }
@@ -1026,13 +1039,14 @@ bool ObjectPrototype::new_enumerate_impl(JSContext* cx, JS::HandleObject obj,
         // Properties
         int n_properties = g_interface_info_get_n_properties(iface_info);
         for (int i = 0; i < n_properties; i++) {
-            GjsAutoInfo<GIPropertyInfo> prop_info =
+            GjsAutoPropertyInfo prop_info =
                 g_interface_info_get_property(iface_info, i);
 
             GjsAutoChar js_name = gjs_hyphen_to_underscore(prop_info.name());
 
             if (!properties.append(gjs_intern_string_to_id(cx, js_name))) {
-                g_error("Unable to append to vector");
+                JS_ReportOutOfMemory(cx);
+                return false;
             }
         }
     }
@@ -1043,14 +1057,14 @@ bool ObjectPrototype::new_enumerate_impl(JSContext* cx, JS::HandleObject obj,
         // Methods
         int n_methods = g_object_info_get_n_methods(info());
         for (int i = 0; i < n_methods; i++) {
-            GjsAutoInfo<GIFunctionInfo> meth_info =
-                g_object_info_get_method(info(), i);
+            GjsAutoFunctionInfo meth_info = g_object_info_get_method(info(), i);
             GIFunctionInfoFlags flags = g_function_info_get_flags(meth_info);
 
             if (flags & GI_FUNCTION_IS_METHOD) {
                 const char* name = meth_info.name();
                 if (!properties.append(gjs_intern_string_to_id(cx, name))) {
-                    g_error("Unable to append to vector");
+                    JS_ReportOutOfMemory(cx);
+                    return false;
                 }
             }
         }
@@ -1058,13 +1072,14 @@ bool ObjectPrototype::new_enumerate_impl(JSContext* cx, JS::HandleObject obj,
         // Properties
         int n_properties = g_object_info_get_n_properties(info());
         for (int i = 0; i < n_properties; i++) {
-            GjsAutoInfo<GIPropertyInfo> prop_info =
+            GjsAutoPropertyInfo prop_info =
                 g_object_info_get_property(info(), i);
 
             GjsAutoChar js_name = gjs_hyphen_to_underscore(prop_info.name());
 
             if (!properties.append(gjs_intern_string_to_id(cx, js_name))) {
-                g_error("Unable to append to vector");
+                JS_ReportOutOfMemory(cx);
+                return false;
             }
         }
     }
@@ -1210,8 +1225,6 @@ ObjectInstance::toggle_down(void)
      * collected by the GC
      */
     if (wrapper_is_rooted()) {
-        GjsContext *context;
-
         debug_lifecycle("Unrooting wrapper");
         switch_to_unrooted();
 
@@ -1231,9 +1244,9 @@ ObjectInstance::toggle_down(void)
          * always queue a garbage collection when a toggle reference goes
          * down.
          */
-        context = gjs_context_get_current();
-        if (!_gjs_context_destroying(context))
-            _gjs_context_schedule_gc(context);
+        GjsContextPrivate* gjs = GjsContextPrivate::from_current_context();
+        if (!gjs->destroying())
+            gjs->schedule_gc();
     }
 }
 
@@ -1285,10 +1298,9 @@ wrapped_gobj_toggle_notify(gpointer      data,
 {
     bool is_main_thread;
     bool toggle_up_queued, toggle_down_queued;
-    GjsContext *context;
 
-    context = gjs_context_get_current();
-    if (_gjs_context_destroying(context)) {
+    GjsContextPrivate* gjs = GjsContextPrivate::from_current_context();
+    if (gjs->destroying()) {
         /* Do nothing here - we're in the process of disassociating
          * the objects.
          */
@@ -1323,7 +1335,7 @@ wrapped_gobj_toggle_notify(gpointer      data,
      * weak singletons like g_bus_get_sync() objects can see toggle-ups
      * from different threads too.
      */
-    is_main_thread = _gjs_context_get_is_owner_thread(context);
+    is_main_thread = gjs->is_owner_thread();
 
     auto& toggle_queue = ToggleQueue::get_default();
     std::tie(toggle_down_queued, toggle_up_queued) = toggle_queue.is_queued(gobj);
@@ -1421,10 +1433,13 @@ ObjectInstance::ObjectInstance(JSContext* cx, JS::HandleObject object)
     debug_lifecycle("Instance constructor");
 }
 
-ObjectPrototype* ObjectPrototype::new_for_js_object(GIObjectInfo* info,
+ObjectPrototype* ObjectPrototype::new_for_js_object(JSContext* cx,
+                                                    GIObjectInfo* info,
                                                     GType gtype) {
     auto* priv = g_slice_new0(ObjectPrototype);
     new (priv) ObjectPrototype(info, gtype);
+    if (!priv->init(cx))
+        return nullptr;
     return priv;
 }
 
@@ -1435,16 +1450,18 @@ ObjectPrototype::ObjectPrototype(GIObjectInfo* info, GType gtype)
 
     g_type_class_ref(gtype);
 
-    if (!m_property_cache.init())
-        g_error("Out of memory for property cache of %s", type_name());
-
-    if (!m_field_cache.init())
-        g_error("Out of memory for field cache of %s", type_name());
-
     GJS_INC_COUNTER(object_prototype);
     debug_lifecycle("Prototype constructor");
 }
 
+bool ObjectPrototype::init(JSContext* cx) {
+    if (!m_property_cache.init() || !m_field_cache.init()) {
+        JS_ReportOutOfMemory(cx);
+        return false;
+    }
+    return true;
+}
+
 static void
 update_heap_wrapper_weak_pointers(JSContext     *cx,
                                   JSCompartment *compartment,
@@ -1661,8 +1678,9 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(object_instance)
      * might be traced and we will end up dereferencing a null pointer */
     JS_SetPrivate(object, ObjectInstance::new_for_js_object(context, object));
 
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
     if (!gjs_object_require_property(context, object, "GObject instance",
-                                     GJS_STRING_GOBJECT_INIT, &initer))
+                                     atoms.init(), &initer))
         return false;
 
     argv.rval().setUndefined();
@@ -1715,7 +1733,7 @@ void ObjectBase::finalize(JSFreeOp* fop, JSObject* obj) {
 ObjectInstance::~ObjectInstance() {
     debug_lifecycle("Finalize");
 
-    TRACE(GJS_OBJECT_PROXY_FINALIZE(priv, m_gobj, ns(), name()));
+    TRACE(GJS_OBJECT_PROXY_FINALIZE(this, m_gobj, ns(), name()));
 
     invalidate_all_closures();
 
@@ -1810,6 +1828,7 @@ JSObject* gjs_lookup_object_constructor_from_info(JSContext* context,
     return constructor;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static JSObject *
 gjs_lookup_object_prototype_from_info(JSContext    *context,
                                       GIObjectInfo *info,
@@ -1821,20 +1840,21 @@ gjs_lookup_object_prototype_from_info(JSContext    *context,
     if (G_UNLIKELY(!constructor))
         return NULL;
 
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
     JS::RootedObject prototype(context);
     if (!gjs_object_require_property(context, constructor, "constructor object",
-                                     GJS_STRING_PROTOTYPE, &prototype))
+                                     atoms.prototype(), &prototype))
         return NULL;
 
     return prototype;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static JSObject *
 gjs_lookup_object_prototype(JSContext *context,
                             GType      gtype)
 {
-    GjsAutoInfo<GIObjectInfo> info =
-        g_irepository_find_by_gtype(nullptr, gtype);
+    GjsAutoObjectInfo info = g_irepository_find_by_gtype(nullptr, gtype);
     return gjs_lookup_object_prototype_from_info(context, info, gtype);
 }
 
@@ -1894,7 +1914,7 @@ ObjectInstance::connect_impl(JSContext          *context,
     if (!check_gobject_disposed("connect to any signal on"))
         return true;
 
-    GjsAutoJSChar signal_name;
+    JS::UniqueChars signal_name;
     JS::RootedObject callback(context);
     if (!gjs_parse_call_args(context, after ? "connect_after" : "connect", args, "so",
                              "signal name", &signal_name,
@@ -1906,14 +1926,15 @@ ObjectInstance::connect_impl(JSContext          *context,
         return false;
     }
 
-    if (!g_signal_parse_name(signal_name, gtype(), &signal_id, &signal_detail,
-                             true)) {
+    if (!g_signal_parse_name(signal_name.get(), gtype(), &signal_id,
+                             &signal_detail, true)) {
         gjs_throw(context, "No signal '%s' on object '%s'",
                   signal_name.get(), type_name());
         return false;
     }
 
-    closure = gjs_closure_new_for_signal(context, callback, "signal callback", signal_id);
+    closure = gjs_closure_new_for_signal(
+        context, JS_GetObjectFunction(callback), "signal callback", signal_id);
     if (closure == NULL)
         return false;
     associate_closure(context, closure);
@@ -1958,13 +1979,13 @@ ObjectInstance::emit_impl(JSContext          *context,
     if (!check_gobject_disposed("emit any signal on"))
         return true;
 
-    GjsAutoJSChar signal_name;
+    JS::UniqueChars signal_name;
     if (!gjs_parse_call_args(context, "emit", argv, "!s",
                              "signal name", &signal_name))
         return false;
 
-    if (!g_signal_parse_name(signal_name, gtype(), &signal_id, &signal_detail,
-                             false)) {
+    if (!g_signal_parse_name(signal_name.get(), gtype(), &signal_id,
+                             &signal_detail, false)) {
         gjs_throw(context, "No signal '%s' on object '%s'",
                   signal_name.get(), type_name());
         return false;
@@ -2110,7 +2131,7 @@ gjs_object_define_static_methods(JSContext       *context,
     for (i = 0; i < n_methods; i++) {
         GIFunctionInfoFlags flags;
 
-        GjsAutoInfo<GICallableInfo> meth_info =
+        GjsAutoCallableInfo meth_info =
             g_object_info_get_method(object_info, i);
         flags = g_function_info_get_flags (meth_info);
 
@@ -2127,7 +2148,7 @@ gjs_object_define_static_methods(JSContext       *context,
         }
     }
 
-    GjsAutoInfo<GIStructInfo> gtype_struct =
+    GjsAutoStructInfo gtype_struct =
         g_object_info_get_class_struct(object_info);
     if (gtype_struct == NULL)
         return true;  /* not an error? */
@@ -2135,7 +2156,7 @@ gjs_object_define_static_methods(JSContext       *context,
     n_methods = g_struct_info_get_n_methods(gtype_struct);
 
     for (i = 0; i < n_methods; i++) {
-        GjsAutoInfo<GICallableInfo> meth_info =
+        GjsAutoCallableInfo meth_info =
             g_struct_info_get_method(gtype_struct, i);
 
         if (!gjs_define_function(context, constructor, gtype, meth_info))
@@ -2145,19 +2166,6 @@ gjs_object_define_static_methods(JSContext       *context,
     return true;
 }
 
-JS::Symbol*
-ObjectInstance::hook_up_vfunc_symbol(JSContext *cx)
-{
-    if (!hook_up_vfunc_root.initialized()) {
-        JS::RootedString descr(cx,
-            JS_NewStringCopyZ(cx, "__GObject__hook_up_vfunc"));
-        if (!descr)
-            g_error("Out of memory defining internal symbol");
-        hook_up_vfunc_root.init(cx, JS::NewSymbol(cx, descr));
-    }
-    return hook_up_vfunc_root;
-}
-
 bool
 gjs_define_object_class(JSContext              *context,
                         JS::HandleObject        in_object,
@@ -2241,14 +2249,16 @@ gjs_define_object_class(JSContext              *context,
 
     /* Hook_up_vfunc can't be included in gjs_object_instance_proto_funcs
      * because it's a custom symbol. */
-    JS::RootedId hook_up_vfunc(context,
-        SYMBOL_TO_JSID(ObjectInstance::hook_up_vfunc_symbol(context)));
-    if (!JS_DefineFunctionById(context, prototype, hook_up_vfunc,
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
+    if (!JS_DefineFunctionById(context, prototype, atoms.hook_up_vfunc(),
                                &ObjectBase::hook_up_vfunc, 3,
                                GJS_MODULE_PROP_FLAGS))
         return false;
 
-    JS_SetPrivate(prototype, ObjectPrototype::new_for_js_object(info, gtype));
+    auto* priv = ObjectPrototype::new_for_js_object(context, info, gtype);
+    if (!priv)
+        return false;
+    JS_SetPrivate(prototype, priv);
 
     gjs_debug(GJS_DEBUG_GOBJECT, "Defined class for %s (%s), prototype %p, "
               "JSClass %p, in object %p", constructor_name, g_type_name(gtype),
@@ -2260,8 +2270,11 @@ gjs_define_object_class(JSContext              *context,
 
     JS::RootedObject gtype_obj(context,
         gjs_gtype_create_gtype_wrapper(context, gtype));
-    return JS_DefineProperty(context, constructor, "$gtype", gtype_obj,
-                             JSPROP_PERMANENT);
+    if (!gtype_obj)
+        return false;
+
+    return JS_DefinePropertyById(context, constructor, atoms.gtype(), gtype_obj,
+                                 JSPROP_PERMANENT);
 }
 
 JSObject*
@@ -2385,20 +2398,15 @@ ObjectInstance::typecheck_object(JSContext *context,
     return result;
 }
 
-
-static bool
-find_vfunc_info (JSContext *context,
-                 GType implementor_gtype,
-                 GIBaseInfo *vfunc_info,
-                 const char   *vfunc_name,
-                 gpointer *implementor_vtable_ret,
-                 GjsAutoInfo<GIFieldInfo> *field_info_ret)
-{
+GJS_JSAPI_RETURN_CONVENTION
+static bool find_vfunc_info(JSContext* context, GType implementor_gtype,
+                            GIBaseInfo* vfunc_info, const char* vfunc_name,
+                            void** implementor_vtable_ret,
+                            GjsAutoFieldInfo* field_info_ret) {
     GType ancestor_gtype;
     int length, i;
     GIBaseInfo *ancestor_info;
-    GjsAutoInfo<GIStructInfo> struct_info;
-    gpointer implementor_class;
+    GjsAutoStructInfo struct_info;
     bool is_interface;
 
     field_info_ret->reset();
@@ -2409,13 +2417,12 @@ find_vfunc_info (JSContext *context,
 
     is_interface = g_base_info_get_type(ancestor_info) == GI_INFO_TYPE_INTERFACE;
 
-    implementor_class = g_type_class_ref(implementor_gtype);
+    GjsAutoTypeClass<GTypeClass> implementor_class(implementor_gtype);
     if (is_interface) {
         GTypeInstance *implementor_iface_class;
         implementor_iface_class = (GTypeInstance*) g_type_interface_peek(implementor_class,
                                                         ancestor_gtype);
         if (implementor_iface_class == NULL) {
-            g_type_class_unref(implementor_class);
             gjs_throw (context, "Couldn't find GType of implementor of interface %s.",
                        g_type_name(ancestor_gtype));
             return false;
@@ -2429,16 +2436,13 @@ find_vfunc_info (JSContext *context,
         *implementor_vtable_ret = implementor_class;
     }
 
-    g_type_class_unref(implementor_class);
-
     length = g_struct_info_get_n_fields(struct_info);
     for (i = 0; i < length; i++) {
-        GjsAutoInfo<GIFieldInfo> field_info =
-            g_struct_info_get_field(struct_info, i);
+        GjsAutoFieldInfo field_info = g_struct_info_get_field(struct_info, i);
         if (strcmp(field_info.name(), vfunc_name) != 0)
             continue;
 
-        GjsAutoInfo<GITypeInfo> type_info = g_field_info_get_type(field_info);
+        GjsAutoTypeInfo type_info = g_field_info_get_type(field_info);
         if (g_type_info_get_tag(type_info) != GI_TYPE_TAG_INTERFACE) {
             /* We have a field with the same name, but it's not a callback.
              * There's no hope of being another field with a correct name,
@@ -2465,7 +2469,7 @@ bool ObjectBase::hook_up_vfunc(JSContext* cx, unsigned argc, JS::Value* vp) {
 bool ObjectPrototype::hook_up_vfunc_impl(JSContext* cx,
                                          const JS::CallArgs& args,
                                          JS::HandleObject prototype) {
-    GjsAutoJSChar name;
+    JS::UniqueChars name;
     JS::RootedObject function(cx);
     if (!gjs_parse_call_args(cx, "hook_up_vfunc", args, "so",
                              "name", &name,
@@ -2480,14 +2484,14 @@ bool ObjectPrototype::hook_up_vfunc_impl(JSContext* cx,
     while (!info && info_gtype != G_TYPE_OBJECT) {
         info_gtype = g_type_parent(info_gtype);
 
-        info = g_irepository_find_by_gtype(g_irepository_get_default(), info_gtype);
+        info = g_irepository_find_by_gtype(nullptr, info_gtype);
     }
 
     /* If we don't have 'info', we don't have the base class (GObject).
      * This is awful, so abort now. */
     g_assert(info != NULL);
 
-    GjsAutoInfo<GIVFuncInfo> vfunc = find_vfunc_on_parents(info, name, nullptr);
+    GjsAutoVFuncInfo vfunc = find_vfunc_on_parents(info, name.get(), nullptr);
 
     if (!vfunc) {
         guint i, n_interfaces;
@@ -2496,13 +2500,13 @@ bool ObjectPrototype::hook_up_vfunc_impl(JSContext* cx,
         interface_list = g_type_interfaces(m_gtype, &n_interfaces);
 
         for (i = 0; i < n_interfaces; i++) {
-            GjsAutoInfo<GIInterfaceInfo> interface =
+            GjsAutoInterfaceInfo interface =
                 g_irepository_find_by_gtype(nullptr, interface_list[i]);
 
             /* The interface doesn't have to exist -- it could be private
              * or dynamic. */
             if (interface) {
-                vfunc = g_interface_info_find_vfunc(interface, name);
+                vfunc = g_interface_info_find_vfunc(interface, name.get());
 
                 if (vfunc)
                     break;
@@ -2519,8 +2523,8 @@ bool ObjectPrototype::hook_up_vfunc_impl(JSContext* cx,
     }
 
     void *implementor_vtable;
-    GjsAutoInfo<GIFieldInfo> field_info;
-    if (!find_vfunc_info(cx, m_gtype, vfunc, name, &implementor_vtable,
+    GjsAutoFieldInfo field_info;
+    if (!find_vfunc_info(cx, m_gtype, vfunc, name.get(), &implementor_vtable,
                          &field_info))
         return false;
 
@@ -2532,9 +2536,13 @@ bool ObjectPrototype::hook_up_vfunc_impl(JSContext* cx,
         offset = g_field_info_get_offset(field_info);
         method_ptr = G_STRUCT_MEMBER_P(implementor_vtable, offset);
 
-        JS::RootedValue v_function(cx, JS::ObjectValue(*function));
+        if (!JS_ObjectIsFunction(cx, function)) {
+            gjs_throw(cx, "Tried to deal with a vfunc that wasn't a function");
+            return false;
+        }
+        JS::RootedFunction func(cx, JS_GetObjectFunction(function));
         trampoline = gjs_callback_trampoline_new(
-            cx, v_function, vfunc, GI_SCOPE_TYPE_NOTIFIED, prototype, true);
+            cx, func, vfunc, GI_SCOPE_TYPE_NOTIFIED, prototype, true);
 
         *((ffi_closure **)method_ptr) = trampoline->closure;
     }
@@ -2549,10 +2557,7 @@ gjs_lookup_object_constructor(JSContext             *context,
 {
     JSObject *constructor;
 
-    GjsAutoInfo<GIObjectInfo> object_info =
-        g_irepository_find_by_gtype(nullptr, gtype);
-
-    g_assert(!object_info || object_info.type() == GI_INFO_TYPE_OBJECT);
+    GjsAutoObjectInfo object_info = g_irepository_find_by_gtype(nullptr, gtype);
 
     constructor = gjs_lookup_object_constructor_from_info(context, object_info, gtype);
 
@@ -2563,15 +2568,11 @@ gjs_lookup_object_constructor(JSContext             *context,
     return true;
 }
 
-bool
-gjs_object_associate_closure(JSContext       *cx,
-                             JS::HandleObject object,
-                             GClosure        *closure)
-{
+void gjs_object_associate_closure(JSContext* cx, JS::HandleObject object,
+                                  GClosure* closure) {
     auto* priv = ObjectBase::for_js(cx, object);
     if (!priv)
-        return false;
+        return;
 
     priv->associate_closure(cx, closure);
-    return true;
 }
