@@ -54,6 +54,7 @@ GJS_DEFINE_PRIV_FROM_JS(Union, gjs_union_class)
  * The *resolved out parameter, on success, should be false to indicate that id
  * was not resolved; and true if id was resolved.
  */
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 union_resolve(JSContext       *context,
               JS::HandleObject obj,
@@ -79,47 +80,39 @@ union_resolve(JSContext       *context,
         return true;
     }
 
-    GjsAutoJSChar name;
-    if (!gjs_get_string_id(context, id, &name)) {
+    JS::UniqueChars name;
+    if (!gjs_get_string_id(context, id, &name))
+        return false;
+    if (!name) {
         *resolved = false;
         return true; /* not resolved, but no error */
     }
 
     /* We are the prototype, so look for methods and other class properties */
-    GIFunctionInfo *method_info;
-
-    method_info = g_union_info_find_method((GIUnionInfo*) priv->info,
-                                           name);
-
-    if (method_info != NULL) {
-        const char *method_name;
+    GjsAutoFunctionInfo method_info =
+        g_union_info_find_method(priv->info, name.get());
 
+    if (method_info) {
 #if GJS_VERBOSE_ENABLE_GI_USAGE
-        _gjs_log_info_usage((GIBaseInfo*) method_info);
+        _gjs_log_info_usage(method_info);
 #endif
         if (g_function_info_get_flags (method_info) & GI_FUNCTION_IS_METHOD) {
-            method_name = g_base_info_get_name( (GIBaseInfo*) method_info);
-
             gjs_debug(GJS_DEBUG_GBOXED,
                       "Defining method %s in prototype for %s.%s",
-                      method_name,
+                      method_info.name(),
                       g_base_info_get_namespace( (GIBaseInfo*) priv->info),
                       g_base_info_get_name( (GIBaseInfo*) priv->info));
 
             /* obj is union proto */
-            if (gjs_define_function(context, obj,
-                                    g_registered_type_info_get_g_type(priv->info),
-                                    method_info) == NULL) {
-                g_base_info_unref( (GIBaseInfo*) method_info);
+            if (!gjs_define_function(
+                    context, obj, g_registered_type_info_get_g_type(priv->info),
+                    method_info))
                 return false;
-            }
 
             *resolved = true; /* we defined the prop in object_proto */
         } else {
             *resolved = false;
         }
-
-        g_base_info_unref( (GIBaseInfo*) method_info);
     } else {
         *resolved = false;
     }
@@ -127,6 +120,7 @@ union_resolve(JSContext       *context,
     return true;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static void*
 union_new(JSContext       *context,
           JS::HandleObject obj, /* "this" for constructor */
@@ -140,10 +134,9 @@ union_new(JSContext       *context,
     n_methods = g_union_info_get_n_methods(info);
 
     for (i = 0; i < n_methods; ++i) {
-        GIFunctionInfo *func_info;
         GIFunctionInfoFlags flags;
 
-        func_info = g_union_info_get_method(info, i);
+        GjsAutoFunctionInfo func_info = g_union_info_get_method(info, i);
 
         flags = g_function_info_get_flags(func_info);
         if ((flags & GI_FUNCTION_IS_CONSTRUCTOR) != 0 &&
@@ -151,24 +144,26 @@ union_new(JSContext       *context,
 
             JS::RootedValue rval(context, JS::NullValue());
 
-            gjs_invoke_c_function_uncached(context, func_info, obj,
-                                           JS::HandleValueArray::empty(), &rval);
-
-            g_base_info_unref((GIBaseInfo*) func_info);
+            if (!gjs_invoke_c_function_uncached(context, func_info, obj,
+                                                JS::HandleValueArray::empty(),
+                                                &rval))
+                return nullptr;
 
             /* We are somewhat wasteful here; invoke_c_function() above
              * creates a JSObject wrapper for the union that we immediately
              * discard.
              */
             if (rval.isNull()) {
+                gjs_throw(context,
+                          "Unable to construct union type %s as its"
+                          "constructor function returned NULL",
+                          g_base_info_get_name(info));
                 return NULL;
             } else {
                 JS::RootedObject rval_obj(context, &rval.toObject());
                 return gjs_c_union_from_union(context, rval_obj);
             }
         }
-
-        g_base_info_unref((GIBaseInfo*) func_info);
     }
 
     gjs_throw(context, "Unable to construct union type %s since it has no zero-args <constructor>, can only wrap an existing one",
@@ -270,6 +265,7 @@ union_finalize(JSFreeOp *fop,
     g_slice_free(Union, priv);
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 to_string_func(JSContext *context,
                unsigned   argc,
@@ -349,7 +345,7 @@ gjs_define_union_class(JSContext       *context,
                                 NULL,
                                 &prototype,
                                 &constructor)) {
-        g_error("Can't init class %s", constructor_name);
+        return false;
     }
 
     GJS_INC_COUNTER(boxed);
@@ -363,11 +359,14 @@ gjs_define_union_class(JSContext       *context,
               constructor_name, prototype.get(), JS_GetClass(prototype),
               in_object.get());
 
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
     JS::RootedObject gtype_obj(context,
         gjs_gtype_create_gtype_wrapper(context, gtype));
-    JS_DefineProperty(context, constructor, "$gtype", gtype_obj, JSPROP_PERMANENT);
+    if (!gtype_obj)
+        return false;
 
-    return true;
+    return JS_DefinePropertyById(context, constructor, atoms.gtype(), gtype_obj,
+                                 JSPROP_PERMANENT);
 }
 
 JSObject*
@@ -397,6 +396,8 @@ gjs_union_from_c_union(JSContext    *context,
 
     JS::RootedObject proto(context,
         gjs_lookup_generic_prototype(context, (GIUnionInfo*) info));
+    if (!proto)
+        return nullptr;
 
     obj = JS_NewObjectWithGivenProto(context, JS_GetClass(proto), proto);
 
