@@ -24,22 +24,23 @@
 
 #include <config.h>
 
+#include <math.h>
+#include <string.h>
+
 #include <codecvt>
 #include <locale>
 
-#include <util/log.h>
-#include <util/glib.h>
-#include <util/misc.h>
-#include <util/error.h>
-
-#include "jsapi-class.h"
-#include "jsapi-util.h"
 #include "jsapi-wrapper.h"
-#include "context-private.h"
-#include <gi/boxed.h>
+#include <js/GCAPI.h>
+#include <js/Printf.h>
 
-#include <string.h>
-#include <math.h>
+#include "gjs/context-private.h"
+#include "gjs/jsapi-class.h"
+#include "gjs/jsapi-util.h"
+#include "util/error.h"
+#include "util/glib.h"
+#include "util/log.h"
+#include "util/misc.h"
 
 GQuark
 gjs_util_error_quark (void)
@@ -47,87 +48,6 @@ gjs_util_error_quark (void)
     return g_quark_from_static_string ("gjs-util-error-quark");
 }
 
-bool
-gjs_object_get_property(JSContext             *cx,
-                        JS::HandleObject       obj,
-                        GjsConstString         property_name,
-                        JS::MutableHandleValue value_p)
-{
-    return JS_GetPropertyById(cx, obj,
-                              gjs_context_get_const_string(cx, property_name),
-                              value_p);
-}
-
-bool
-gjs_object_set_property(JSContext       *cx,
-                        JS::HandleObject obj,
-                        GjsConstString   property_name,
-                        JS::HandleValue  value)
-{
-    return JS_SetPropertyById(cx, obj,
-                              gjs_context_get_const_string(cx, property_name),
-                              value);
-}
-
-bool
-gjs_object_has_property(JSContext       *cx,
-                        JS::HandleObject obj,
-                        GjsConstString   property_name,
-                        bool            *found)
-{
-    return JS_HasPropertyById(cx, obj,
-                              gjs_context_get_const_string(cx, property_name),
-                              found);
-}
-
-bool
-gjs_object_define_property(JSContext       *cx,
-                           JS::HandleObject obj,
-                           GjsConstString   property_name,
-                           JS::HandleValue  value,
-                           unsigned         flags)
-{
-    return JS_DefinePropertyById(cx, obj,
-                                 gjs_context_get_const_string(cx, property_name),
-                                 value, flags);
-}
-
-bool
-gjs_object_define_property(JSContext       *cx,
-                           JS::HandleObject obj,
-                           GjsConstString   property_name,
-                           JS::HandleObject value,
-                           unsigned         flags)
-{
-    return JS_DefinePropertyById(cx, obj,
-                                 gjs_context_get_const_string(cx, property_name),
-                                 value, flags);
-}
-
-bool
-gjs_object_define_property(JSContext       *cx,
-                           JS::HandleObject obj,
-                           GjsConstString   property_name,
-                           JS::HandleString value,
-                           unsigned         flags)
-{
-    return JS_DefinePropertyById(cx, obj,
-                                 gjs_context_get_const_string(cx, property_name),
-                                 value, flags);
-}
-
-bool
-gjs_object_define_property(JSContext       *cx,
-                           JS::HandleObject obj,
-                           GjsConstString   property_name,
-                           uint32_t         value,
-                           unsigned         flags)
-{
-    return JS_DefinePropertyById(cx, obj,
-                                 gjs_context_get_const_string(cx, property_name),
-                                 value, flags);
-}
-
 static void
 throw_property_lookup_error(JSContext       *cx,
                             JS::HandleObject obj,
@@ -138,15 +58,12 @@ throw_property_lookup_error(JSContext       *cx,
     /* remember gjs_throw() is a no-op if JS_GetProperty()
      * already set an exception
      */
-    GjsAutoJSChar name;
-    gjs_get_string_id(cx, property_name, &name);
-
     if (description)
-        gjs_throw(cx, "No property '%s' in %s (or %s)", name.get(), description,
-                  reason);
+        gjs_throw(cx, "No property '%s' in %s (or %s)",
+                  gjs_debug_id(property_name).c_str(), description, reason);
     else
-        gjs_throw(cx, "No property '%s' in object %p (or %s)", name.get(),
-                  obj.address(), reason);
+        gjs_throw(cx, "No property '%s' in object %p (or %s)",
+                  gjs_debug_id(property_name).c_str(), obj.get(), reason);
 }
 
 /* Returns whether the object had the property; if the object did
@@ -219,13 +136,10 @@ gjs_object_require_property(JSContext       *cx,
 }
 
 /* Converts JS string value to UTF-8 string. value must be freed with JS_free. */
-bool
-gjs_object_require_property(JSContext       *cx,
-                            JS::HandleObject obj,
-                            const char      *description,
-                            JS::HandleId     property_name,
-                            GjsAutoJSChar   *value)
-{
+bool gjs_object_require_property(JSContext* cx, JS::HandleObject obj,
+                                 const char* description,
+                                 JS::HandleId property_name,
+                                 JS::UniqueChars* value) {
     JS::RootedValue prop_value(cx);
     if (JS_GetPropertyById(cx, obj, property_name, &prop_value) &&
         gjs_string_to_utf8(cx, prop_value, value))
@@ -287,10 +201,10 @@ gjs_throw_abstract_constructor_error(JSContext    *context,
     const JSClass *proto_class;
     const char *name = "anonymous";
 
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
     JS::RootedObject callee(context, &args.callee());
     JS::RootedValue prototype(context);
-    if (gjs_object_get_property(context, callee, GJS_STRING_PROTOTYPE,
-                                &prototype)) {
+    if (JS_GetPropertyById(context, callee, atoms.prototype(), &prototype)) {
         proto_class = JS_GetClass(&prototype.toObject());
         name = proto_class->name;
     }
@@ -309,15 +223,16 @@ gjs_build_string_array(JSContext   *context,
         array_length = g_strv_length(array_values);
 
     JS::AutoValueVector elems(context);
-    if (!elems.reserve(array_length))
-        g_error("Unable to reserve memory for vector");
+    if (!elems.reserve(array_length)) {
+        JS_ReportOutOfMemory(context);
+        return nullptr;
+    }
 
     for (i = 0; i < array_length; ++i) {
         JS::ConstUTF8CharsZ chars(array_values[i], strlen(array_values[i]));
         JS::RootedValue element(context,
             JS::StringValue(JS_NewStringCopyUTF8Z(context, chars)));
-        if (!elems.append(element))
-            g_error("Unable to append to vector");
+        elems.infallibleAppend(element);
     }
 
     return JS_NewArrayObject(context, elems);
@@ -354,6 +269,7 @@ gjs_define_string_array(JSContext       *context,
  * are \x escaped.
  *
  */
+GJS_USE
 static char *
 gjs_string_readable(JSContext       *context,
                     JS::HandleString string)
@@ -364,7 +280,7 @@ gjs_string_readable(JSContext       *context,
 
     g_string_append_c(buf, '"');
 
-    GjsAutoJSChar chars = JS_EncodeStringToUTF8(context, string);
+    JS::UniqueChars chars(JS_EncodeStringToUTF8(context, string));
     if (!chars) {
         /* I'm not sure this code will actually ever be reached except in the
          * case of OOM, since JS_EncodeStringToUTF8() seems to happily output
@@ -379,7 +295,7 @@ gjs_string_readable(JSContext       *context,
         g_string_append(buf, escaped);
         g_free(escaped);
     } else {
-        g_string_append(buf, chars);
+        g_string_append(buf, chars.get());
     }
 
     g_string_append_c(buf, '"');
@@ -389,6 +305,7 @@ gjs_string_readable(JSContext       *context,
     return g_string_free(buf, false);
 }
 
+GJS_USE
 static char *
 _gjs_g_utf8_make_valid (const char *name)
 {
@@ -452,6 +369,11 @@ gjs_value_debug_string(JSContext      *context,
 
     JS::RootedString str(context, JS::ToString(context, value));
 
+    if (!str) {
+        JS_ClearPendingException(context);
+        str = JS_ValueToSource(context, value);
+    }
+
     if (!str) {
         if (value.isObject()) {
             /* Specifically the Call object (see jsfun.c in spidermonkey)
@@ -487,52 +409,34 @@ gjs_value_debug_string(JSContext      *context,
     return debugstr;
 }
 
-static char *
-utf8_exception_from_non_gerror_value(JSContext      *cx,
-                                     JS::HandleValue exc)
-{
-    JS::RootedString exc_str(cx, JS::ToString(cx, exc));
-    if (!exc_str)
-        return nullptr;
-
-    GjsAutoJSChar utf8_exception = JS_EncodeStringToUTF8(cx, exc_str);
-    return utf8_exception.copy();
-}
-
 bool
 gjs_log_exception_full(JSContext       *context,
                        JS::HandleValue  exc,
                        JS::HandleString message)
 {
-    char *utf8_exception;
     bool is_syntax;
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
 
     JS_BeginRequest(context);
     JS::RootedObject exc_obj(context);
+    JS::RootedString exc_str(context, JS::ToString(context, exc));
+    JS::UniqueChars utf8_exception;
+    if (exc_str)
+        utf8_exception.reset(JS_EncodeStringToUTF8(context, exc_str));
+    if (!utf8_exception)
+        JS_ClearPendingException(context);
 
     is_syntax = false;
-
-    if (!exc.isObject()) {
-        utf8_exception = utf8_exception_from_non_gerror_value(context, exc);
-    } else {
+    if (exc.isObject()) {
         exc_obj = &exc.toObject();
-        if (gjs_typecheck_boxed(context, exc_obj, NULL, G_TYPE_ERROR, false)) {
-            GError *gerror = (GError *) gjs_c_struct_from_boxed(context, exc_obj);
-            utf8_exception = g_strdup_printf("GLib.Error %s: %s",
-                                             g_quark_to_string(gerror->domain),
-                                             gerror->message);
-        } else {
-            const JSClass *syntax_error =
-                js::Jsvalify(js::ProtoKeyToClass(JSProto_SyntaxError));
-            is_syntax = JS_InstanceOf(context, exc_obj, syntax_error, nullptr);
-
-            utf8_exception = utf8_exception_from_non_gerror_value(context, exc);
-        }
+        const JSClass* syntax_error =
+            js::Jsvalify(js::ProtoKeyToClass(JSProto_SyntaxError));
+        is_syntax = JS_InstanceOf(context, exc_obj, syntax_error, nullptr);
     }
 
-    GjsAutoJSChar utf8_message;
+    JS::UniqueChars utf8_message;
     if (message)
-        utf8_message = JS_EncodeStringToUTF8(context, message);
+        utf8_message.reset(JS_EncodeStringToUTF8(context, message));
 
     /* We log syntax errors differently, because the stack for those includes
        only the referencing module, but we want to print out the filename and
@@ -543,56 +447,55 @@ gjs_log_exception_full(JSContext       *context,
         JS::RootedValue js_lineNumber(context), js_fileName(context);
         unsigned lineNumber;
 
-        gjs_object_get_property(context, exc_obj, GJS_STRING_LINE_NUMBER,
-                                &js_lineNumber);
-        gjs_object_get_property(context, exc_obj, GJS_STRING_FILENAME,
-                                &js_fileName);
+        JS_GetPropertyById(context, exc_obj, atoms.line_number(),
+                           &js_lineNumber);
+        JS_GetPropertyById(context, exc_obj, atoms.file_name(), &js_fileName);
 
-        GjsAutoJSChar utf8_filename;
+        JS::UniqueChars utf8_filename;
         if (js_fileName.isString()) {
             JS::RootedString str(context, js_fileName.toString());
-            utf8_filename = JS_EncodeStringToUTF8(context, str);
+            utf8_filename.reset(JS_EncodeStringToUTF8(context, str));
         }
         if (!utf8_filename)
-            utf8_filename = JS_strdup(context, "unknown");
+            utf8_filename.reset(JS_strdup(context, "unknown"));
 
         lineNumber = js_lineNumber.toInt32();
 
         if (message) {
-            g_critical("JS ERROR: %s: %s @ %s:%u", utf8_message.get(), utf8_exception,
-                       utf8_filename.get(), lineNumber);
+            g_critical("JS ERROR: %s: %s @ %s:%u", utf8_message.get(),
+                       utf8_exception.get(), utf8_filename.get(), lineNumber);
         } else {
-            g_critical("JS ERROR: %s @ %s:%u", utf8_exception,
+            g_critical("JS ERROR: %s @ %s:%u", utf8_exception.get(),
                        utf8_filename.get(), lineNumber);
         }
 
     } else {
-        GjsAutoJSChar utf8_stack;
+        JS::UniqueChars utf8_stack;
         JS::RootedValue stack(context);
 
         if (exc.isObject() &&
-            gjs_object_get_property(context, exc_obj, GJS_STRING_STACK,
-                                    &stack) &&
+            JS_GetPropertyById(context, exc_obj, atoms.stack(), &stack) &&
             stack.isString()) {
             JS::RootedString str(context, stack.toString());
-            utf8_stack = JS_EncodeStringToUTF8(context, str);
+            utf8_stack.reset(JS_EncodeStringToUTF8(context, str));
         }
 
         if (message) {
             if (utf8_stack)
-                g_warning("JS ERROR: %s: %s\n%s", utf8_message.get(), utf8_exception, utf8_stack.get());
+                g_warning("JS ERROR: %s: %s\n%s", utf8_message.get(),
+                          utf8_exception.get(), utf8_stack.get());
             else
-                g_warning("JS ERROR: %s: %s", utf8_message.get(), utf8_exception);
+                g_warning("JS ERROR: %s: %s", utf8_message.get(),
+                          utf8_exception.get());
         } else {
             if (utf8_stack)
-                g_warning("JS ERROR: %s\n%s", utf8_exception, utf8_stack.get());
+                g_warning("JS ERROR: %s\n%s", utf8_exception.get(),
+                          utf8_stack.get());
             else
-                g_warning("JS ERROR: %s", utf8_exception);
+                g_warning("JS ERROR: %s", utf8_exception.get());
         }
     }
 
-    g_free(utf8_exception);
-
     JS_EndRequest(context);
 
     return true;
@@ -641,29 +544,6 @@ gjs_call_function_value(JSContext                  *context,
     return result;
 }
 
-/* get a debug string for type tag in JS::Value */
-const char*
-gjs_get_type_name(JS::Value value)
-{
-    if (value.isNull()) {
-        return "null";
-    } else if (value.isUndefined()) {
-        return "undefined";
-    } else if (value.isInt32()) {
-        return "integer";
-    } else if (value.isDouble()) {
-        return "double";
-    } else if (value.isBoolean()) {
-        return "boolean";
-    } else if (value.isString()) {
-        return "string";
-    } else if (value.isObject()) {
-        return "object";
-    } else {
-        return "<unknown>";
-    }
-}
-
 #ifdef __linux__
 static void
 _linux_get_self_process_size (gulong *vm_size,
@@ -732,7 +612,7 @@ gjs_gc_if_needed (JSContext *context)
          */
         if (rss_size > linux_rss_trigger) {
             linux_rss_trigger = (gulong) MIN(G_MAXULONG, rss_size * 1.25);
-            JS_GC(context);
+            JS::GCForReason(context, GC_SHRINK, JS::gcreason::Reason::API);
         } else if (rss_size < (0.75 * linux_rss_trigger)) {
             /* If we've shrunk by 75%, lower the trigger */
             linux_rss_trigger = (rss_size * 1.25);
@@ -756,16 +636,12 @@ gjs_maybe_gc (JSContext *context)
 void
 gjs_schedule_gc_if_needed (JSContext *context)
 {
-    GjsContext *gjs_context;
-
     /* We call JS_MaybeGC immediately, but defer a check for a full
      * GC cycle to an idle handler.
      */
     JS_MaybeGC(context);
 
-    gjs_context = (GjsContext *) JS_GetContextPrivate(context);
-    if (gjs_context)
-        _gjs_context_schedule_gc_if_needed(gjs_context);
+    GjsContextPrivate::from_cx(context)->schedule_gc_if_needed();
 }
 
 /**
@@ -852,8 +728,7 @@ gjs_eval_with_scope(JSContext             *context,
         eval_obj = JS_NewPlainObject(context);
 
     JS::CompileOptions options(context);
-    options.setFileAndLine(filename, start_line_number)
-           .setSourceIsLazy(true);
+    options.setFileAndLine(filename, start_line_number);
 
     std::wstring_convert<std::codecvt_utf8_utf16<char16_t>, char16_t> convert;
     std::u16string utf16_string = convert.from_bytes(script);
@@ -861,8 +736,10 @@ gjs_eval_with_scope(JSContext             *context,
                                JS::SourceBufferHolder::NoOwnership);
 
     JS::AutoObjectVector scope_chain(context);
-    if (!scope_chain.append(eval_obj))
-        g_error("Unable to append to vector");
+    if (!scope_chain.append(eval_obj)) {
+        JS_ReportOutOfMemory(context);
+        return false;
+    }
 
     if (!JS::Evaluate(context, scope_chain, options, buf, retval))
         return false;
