@@ -38,8 +38,9 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "config.h"
+#include <config.h>
 
+#include <mozilla/Unused.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -59,47 +60,93 @@
 #include "gjs/context-private.h"
 #include "gjs/jsapi-wrapper.h"
 
+enum class PrintErrorKind { Error, Warning, StrictWarning, Note };
+
+template <typename T>
+static void print_single_error(T* report, PrintErrorKind kind);
+static void print_error_line(const char* prefix, JSErrorNotes::Note* note) {}
+static void print_error_line(const char* prefix, JSErrorReport* report);
+
 static void
 gjs_console_print_error(JSErrorReport *report)
 {
-    /* Code modified from SpiderMonkey js/src/jscntxt.cpp, js::PrintError() */
+    // Code modified from SpiderMonkey js/src/vm/JSContext.cpp, js::PrintError()
 
     g_assert(report);
 
-    char *prefix = nullptr;
-    if (report->filename)
-        prefix = g_strdup_printf("%s:", report->filename);
-    if (report->lineno) {
-        char *tmp = prefix;
-        prefix = g_strdup_printf("%s%u:%u ", tmp ? tmp : "", report->lineno,
-                                 report->column);
-        g_free(tmp);
-    }
+    PrintErrorKind kind = PrintErrorKind::Error;
     if (JSREPORT_IS_WARNING(report->flags)) {
-        char *tmp = prefix;
-        prefix = g_strdup_printf("%s%swarning: ",
-                                 tmp ? tmp : "",
-                                 JSREPORT_IS_STRICT(report->flags) ? "strict " : "");
-        g_free(tmp);
+        if (JSREPORT_IS_STRICT(report->flags))
+            kind = PrintErrorKind::StrictWarning;
+        else
+            kind = PrintErrorKind::Warning;
+    }
+    print_single_error(report, kind);
+
+    if (report->notes) {
+        for (auto&& note : *report->notes)
+            print_single_error(note.get(), PrintErrorKind::Note);
     }
 
-    const char *message = report->message().c_str();
+    return;
+}
+
+template <typename T>
+static void print_single_error(T* report, PrintErrorKind kind) {
+    JS::UniqueChars prefix;
+    if (report->filename)
+        prefix.reset(g_strdup_printf("%s:", report->filename));
+
+    if (report->lineno) {
+        prefix.reset(g_strdup_printf("%s%u:%u ", prefix ? prefix.get() : "",
+                                     report->lineno, report->column));
+    }
+
+    if (kind != PrintErrorKind::Error) {
+        const char* kindPrefix = nullptr;
+        switch (kind) {
+            case PrintErrorKind::Warning:
+                kindPrefix = "warning";
+                break;
+            case PrintErrorKind::StrictWarning:
+                kindPrefix = "strict warning";
+                break;
+            case PrintErrorKind::Note:
+                kindPrefix = "note";
+                break;
+            case PrintErrorKind::Error:
+            default:
+                g_assert_not_reached();
+        }
+
+        prefix.reset(
+            g_strdup_printf("%s%s: ", prefix ? prefix.get() : "", kindPrefix));
+    }
+
+    const char* message = report->message().c_str();
 
     /* embedded newlines -- argh! */
     const char *ctmp;
     while ((ctmp = strchr(message, '\n')) != 0) {
         ctmp++;
         if (prefix)
-            fputs(prefix, stderr);
-        fwrite(message, 1, ctmp - message, stderr);
+            fputs(prefix.get(), stderr);
+        mozilla::Unused << fwrite(message, 1, ctmp - message, stderr);
         message = ctmp;
     }
 
     /* If there were no filename or lineno, the prefix might be empty */
     if (prefix)
-        fputs(prefix, stderr);
+        fputs(prefix.get(), stderr);
     fputs(message, stderr);
 
+    print_error_line(prefix.get(), report);
+    fputc('\n', stderr);
+
+    fflush(stderr);
+}
+
+static void print_error_line(const char* prefix, JSErrorReport* report) {
     if (const char16_t* linebuf = report->linebuf()) {
         size_t n = report->linebufLength();
 
@@ -129,9 +176,6 @@ gjs_console_print_error(JSErrorReport *report)
         }
         fputc('^', stderr);
     }
-    fputc('\n', stderr);
-    fflush(stderr);
-    g_free(prefix);
 }
 
 static void
@@ -193,6 +237,7 @@ public:
 };
 
 #ifdef HAVE_READLINE_READLINE_H
+GJS_USE
 static bool
 gjs_console_readline(JSContext *cx, char **bufp, FILE *file, const char *prompt)
 {
@@ -206,6 +251,7 @@ gjs_console_readline(JSContext *cx, char **bufp, FILE *file, const char *prompt)
     return true;
 }
 #else
+GJS_USE
 static bool
 gjs_console_readline(JSContext *cx, char **bufp, FILE *file, const char *prompt)
 {
@@ -223,6 +269,7 @@ gjs_console_readline(JSContext *cx, char **bufp, FILE *file, const char *prompt)
  * exception. (This is because the exception should be auto-printed around the
  * invocation of this function.)
  */
+GJS_USE
 static bool
 gjs_console_eval_and_print(JSContext  *cx,
                            const char *bytes,
@@ -244,10 +291,6 @@ gjs_console_eval_and_print(JSContext  *cx,
     if (result.isUndefined())
         return true;
 
-    JS::RootedString str(cx, JS::ToString(cx, result));
-    if (!str)
-        return true;
-
     char *display_str;
     display_str = gjs_value_debug_string(cx, result);
     if (display_str) {
@@ -257,6 +300,7 @@ gjs_console_eval_and_print(JSContext  *cx,
     return true;
 }
 
+GJS_JSAPI_RETURN_CONVENTION
 static bool
 gjs_console_interact(JSContext *context,
                      unsigned   argc,
@@ -304,8 +348,8 @@ gjs_console_interact(JSContext *context,
         }
         g_string_free(buffer, true);
 
-        auto gjs_context = static_cast<GjsContext *>(JS_GetContextPrivate(context));
-        ok = _gjs_context_run_jobs(gjs_context) && ok;
+        GjsContextPrivate* gjs = GjsContextPrivate::from_cx(context);
+        ok = gjs->run_jobs() && ok;
 
         if (!ok) {
             /* If this was an uncatchable exception, throw another uncatchable
@@ -331,6 +375,8 @@ gjs_define_console_stuff(JSContext              *context,
                          JS::MutableHandleObject module)
 {
     module.set(JS_NewPlainObject(context));
-    return JS_DefineFunction(context, module, "interact", gjs_console_interact,
-                             1, GJS_MODULE_PROP_FLAGS);
+    const GjsAtoms& atoms = GjsContextPrivate::atoms(context);
+    return JS_DefineFunctionById(context, module, atoms.interact(),
+                                 gjs_console_interact, 1,
+                                 GJS_MODULE_PROP_FLAGS);
 }
